; vim: syn=pic
;
; Blinkring - Blinking Earring
; Copyright (C) 2013  B. Stultiens
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;
	list
	errorlevel	1
	radix		dec

	include		"p12lf1552.inc"

	__CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_SWDTEN & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _BOREN_SBODEN & _CLKOUTEN_OFF
	__CONFIG _CONFIG2, _WRT_OFF & _STVREN_ON & _BORV_LO & _LPBOR_OFF & _LVP_ON
	__CONFIG _IDLOC0, 'E'
	__CONFIG _IDLOC1, 'R'
	__CONFIG _IDLOC2, (VERSIONHI + '0')
	__CONFIG _IDLOC3, (VERSIONLO + '0')

; Variables in the access-ram
; The RED/GRN addresses must start on a 16byte boundary
red0		=	0x70	; Red leds intensity
red1		=	0x71
red2		=	0x72
red3		=	0x73
red4		=	0x74
red5		=	0x75
grn0		=	0x76	; Green leds intensity
grn1		=	0x77
grn2		=	0x78
grn3		=	0x79
grn4		=	0x7a
grn5		=	0x7b
pwm		=	0x7c	; PWM counter
mode		=	0x7d	; Current mode of blinking
ixcntl		=	0x7e	; Current index count into running mode
ixcnth		=	0x7f

CMDALLRED	=	0x0c	; All red leds set command
CMDALLGRN	=	0x0d	; All green leds set command
CMDALLYLW	=	0x0e	; All leds set command
CMDNOP		=	0x0f	; Nop command

; These are in bank 0 (where PORTA is located)
current_startl	=	0x20
current_starth	=	0x21
button		=	0x22
tempvar		=	0x23

FOSC		=	1000000
FOSC4		=	FOSC/4
T0CNT		=	-(FOSC4/128/10-2)	; 10Hz

;
; LED positioning:
;     D5  D3
; D4          D1
;     D6  D2
;
; Suffixes used:
;     1   0
;  2         5
;     3   4
;
BUTTONBIT	=	RA2		; Button
D0		=	(1<<RA0)	; Data line D0
D1		=	(1<<RA1)	; Data line D1
D2		=	(1<<RA4)	; Data line D2
D3		=	(1<<RA5)	; Data line D3
nD0		=	0		; Negative datalines
nD1		=	0
nD2		=	0
nD3		=	0
red0val		=	-1^(D3|nD0)
red1val		=	-1^(D3|nD1)
red2val		=	-1^(D2|nD1)
red3val		=	-1^(D3|nD2)
red4val		=	-1^(D2|nD0)
red5val		=	-1^(D1|nD0)
grn0val		=	-1^(nD3|D0)
grn1val		=	-1^(nD3|D1)
grn2val		=	-1^(nD2|D1)
grn3val		=	-1^(nD3|D2)
grn4val		=	-1^(nD2|D0)
grn5val		=	-1^(nD1|D0)
red0io		=	~(D3|D0)
red1io		=	~(D3|D1)
red2io		=	~(D2|D1)
red3io		=	~(D3|D2)
red4io		=	~(D2|D0)
red5io		=	~(D1|D0)
grn0io		=	~(D3|D0)
grn1io		=	~(D3|D1)
grn2io		=	~(D2|D1)
grn3io		=	~(D3|D2)
grn4io		=	~(D2|D0)
grn5io		=	~(D1|D0)

;
; PWM cycle a LED
; If the PWM counter is less than the led value
; then the output is on.
;
; 11 cycles
;
charlicycles	= 12

charlipwm	macro	__led, __val, __io
	bsf	INTCON, GIE	; This window allows interrupts to pass
	nop
	bcf	INTCON, GIE
	movf	__led, w
	subwf	pwm, w
	; (pwm - __led) --> __led > pwm --> carry == 0
	movlw	0xff		; All leds off
	movwi	FSR1		; FSR1 points to TRISA
	movlw	__val
	movwf	PORTA		; Port value is not conditional
	movlw	__io		; Setup new I/O mode
	btfss	STATUS, C
	movwi	FSR1		; Only if PWM says so
	endm

;
; Delay for an exact number of cycles
;
delaycycles	macro	n
dcnt = n
	while dcnt > 0
	nop
dcnt = dcnt - 1
	endw
	endm

;
;------------------------------------------------------------------------------
; Reset entry
;------------------------------------------------------------------------------
	org	0x0000
	goto	_main
	nop
	nop
	nop

;
;------------------------------------------------------------------------------
; Interrupt service
; - handle wake-up via IOC interrupt
;	* restore pull-ups
;	* clear the change condition
;
; - handle timer interrupt
;	* get next command and set the led accordingly
;	* update index counter and wrap as appropriate
;	* check for button input
;		- debounce
;		- advance next mode setup pointers and sizes
;------------------------------------------------------------------------------
;
	org	0x0004
_isr
	; Set port all high, disabling LED(s). This will prevent intensity
	; changes when the timer fires. The isr is only enabled in a small
	; window just before we change to the next LED. Entering the isr
	; takes enough time to complete the last cycle.
	movlw	0xff
	movwf	PORTA

	; Interrupt from input button change (makes us wake-up)
	btfss	INTCON, IOCIF
	goto	no_ioc
	banksel	WPUA			; Restore the pull-ups to normal
	movlw	(1<<WPUA3)|(1<<WPUA2)	; Pull up on switch and reset only
	movwf	WPUA
	banksel	IOCAF
	movlw	0xff			; Reset the change flags
	xorwf	IOCAF, w
	andwf	IOCAF, f
	banksel	PORTA
	bcf	INTCON, IOCIF
	bsf	mode, 7			; This will prevent us to sleep until next timer

no_ioc
	; Check timer
	btfss	INTCON, TMR0IF
	goto	skip_timer

	; Timer fired, setup next led command
	bcf	INTCON, TMR0IF
	bcf	mode, 7			; Make us go to sleep in mode 0 again

	movlw	T0CNT			; Reset counter
	movwf	TMR0

	; Interpret the next command and jump accordingly
	moviw	FSR0
	andlw	0x0f
	movwf	tempvar
	moviw	FSR0++
	swapf	WREG, w
	andlw	0x0f
	brw
	goto	_set_red0
	goto	_set_red1
	goto	_set_red2
	goto	_set_red3
	goto	_set_red4
	goto	_set_red5
	goto	_set_grn0
	goto	_set_grn1
	goto	_set_grn2
	goto	_set_grn3
	goto	_set_grn4
	goto	_set_grn5
	goto	_set_allred
	goto	_set_allgrn
	goto	_set_allylw
	goto	_set_nop
_set_red0
	movf	tempvar, w
	movwf	red0
	goto	_set_done
_set_red1
	movf	tempvar, w
	movwf	red1
	goto	_set_done
_set_red2
	movf	tempvar, w
	movwf	red2
	goto	_set_done
_set_red3
	movf	tempvar, w
	movwf	red3
	goto	_set_done
_set_red4
	movf	tempvar, w
	movwf	red4
	goto	_set_done
_set_red5
	movf	tempvar, w
	movwf	red5
	goto	_set_done
_set_grn0
	movf	tempvar, w
	movwf	grn0
	goto	_set_done
_set_grn1
	movf	tempvar, w
	movwf	grn1
	goto	_set_done
_set_grn2
	movf	tempvar, w
	movwf	grn2
	goto	_set_done
_set_grn3
	movf	tempvar, w
	movwf	grn3
	goto	_set_done
_set_grn4
	movf	tempvar, w
	movwf	grn4
	goto	_set_done
_set_grn5
	movf	tempvar, w
	movwf	grn5
	goto	_set_done
_set_allred
	movf	tempvar, w
	movwf	red0
	movwf	red1
	movwf	red2
	movwf	red3
	movwf	red4
	movwf	red5
	goto	_set_done
_set_allgrn
	movf	tempvar, w
	movwf	grn0
	movwf	grn1
	movwf	grn2
	movwf	grn3
	movwf	grn4
	movwf	grn5
	goto	_set_done
_set_allylw
	movf	tempvar, w
	movwf	red0
	movwf	red1
	movwf	red2
	movwf	red3
	movwf	red4
	movwf	red5
	movwf	grn0
	movwf	grn1
	movwf	grn2
	movwf	grn3
	movwf	grn4
	movwf	grn5
;	goto	_set_done
_set_nop
_set_done
	
	movlw	-1			; Decrement index counter
	addwf	ixcntl, f
	btfss	STATUS, C
	decf	ixcnth, f
	movf	ixcntl, w		; if ixcnt == 0
	iorwf	ixcnth, w
	btfss	STATUS, Z
	goto	not_zero
	movf	mode, f
	goto	reset_mode

not_zero
	; Handle the button
	btfsc	PORTA, BUTTONBIT
	clrf	button			; Reset status if button not pressed

	bsf	STATUS, C		; Set carry to shift into bit 0
	btfss	PORTA, BUTTONBIT
	rlf	button, f		; 0001 -> 0011 -> 0111 -> 1111, etc

	btfss	button, 1		; If 0011, we've seen it twice
	goto	no_button		; otherwise we are in debounce

	btfss	STATUS, C		; If we have 9 ones (pressed for 9 timer ints)
	goto	no_turnoff		; turn off

	clrw
	goto	dosetmode

no_turnoff
	btfsc	button, 2		; If 0111, then we've been here before
	goto	no_button

	; Here we have a debounced button
	; Set new mode.
	movlw	-1			; Decrement mode
	addwf	mode, f
	movlw	(_modes_list_end - _modes_list) / 4 - 1
	btfss	STATUS, C		; Wrap to last
dosetmode
	movwf	mode
	call	_kill_leds
reset_mode
	movf	mode, w
	lslf	WREG, w			; mode * 4
	lslf	WREG, w
	addlw	LOW(_modes_list | 0x8000)
	movwf	FSR0L			; low(base + 4*mode)
if (_modes_list & 0xff00) != ((_modes_list_end-1) & 0xff00)
	clrw
	rlf	WREG, w			; put possible carry in bit0
	addlw	HIGH(_modes_list | 0x8000) ; perform add literal with carry
else
	movlw	HIGH(_modes_list | 0x8000)
endif
	movwf	FSR0H
	moviw	FSR0++			; Get the new address and size
	movwf	current_startl
	moviw	FSR0++
	movwf	current_starth
	moviw	FSR0++
	movwf	ixcntl
	moviw	FSR0++
	movwf	ixcnth
	movf	current_starth, w	; Setup new pointer
	movwf	FSR0H
	movf	current_startl, w
	movwf	FSR0L
	bsf	mode, 7			; Make sure we do not sleep until next timer

no_button
	banksel	FSR0L_SHAD		; Update the shadow register
	movf	FSR0L, w		; or fsr0 changes will have no effect
	movwf	FSR0L_SHAD
	movf	FSR0H, w
	movwf	FSR0H_SHAD
skip_timer
	retfie

;
;------------------------------------------------------------------------------
; Start of program
;------------------------------------------------------------------------------
;
_main
	banksel	OSCCON
	; Internal OSC @ 1MHz
	movlw	(1<<IRCF3)|(1<<IRCF1)|(1<<IRCF0)|(1<<SCS1)
	movwf	OSCCON

	; Enable pull-up, Tmr0 prescaler 1:128 (7.6Hz @ Fosc/4=250kHz)
	movlw	(1<<PS2)|(1<<PS1)
	movwf	OPTION_REG

	banksel	WPUA
	movlw	(1<<WPUA3)|(1<<WPUA2)	; Pull up on switch and reset
	movwf	WPUA
	banksel	LATA
	movwf	LATA
	banksel	ANSELA
	clrf	ANSELA		; All digital I/O
	banksel	TRISA
	movlw	0xff		; All input/Hi-Z
	movwf	TRISA

	banksel	IOCAN
	movlw	(1<<IOCAN2)	; Interrupt on input down flank
	movwf	IOCAN
	clrf	IOCAP
	movlw	0xff		; Reset the change flags
	xorwf	IOCAF, w
	andwf	IOCAF, f

	; Let FSR1 point to TRISA
	movlw	HIGH(TRISA)
	movwf	FSR1H
	movlw	LOW(TRISA)
	movwf	FSR1L

	; Setting the bank here will allow (most of) the rest to do without.
	; It also gives access to an additional 80 bytes of RAM located at 0x20-0x6f.
	banksel	PORTA

	clrf	mode
	clrf	pwm
	call	_kill_leds

	movlw	HIGH(_alloff_table | 0x8000)
	movwf	current_starth
	movwf	FSR0H
	movlw	LOW(_alloff_table | 0x8000)
	movwf	current_startl
	movwf	FSR0L
	movlw	HIGH(_alloff_table_end - _alloff_table)
	movwf	ixcnth
	movlw	LOW(_alloff_table_end - _alloff_table)
	movwf	ixcntl
	clrf	button

	; Enable interrupts
	; The PWM code creates a small window
	movlw	(1<<GIE)|(1<<TMR0IE)|(1<<IOCIE)
	movwf	INTCON

	goto	_loop_entry

;
;------------------------------------------------------------------------------
; Reset all LEDs to off so that the nex mode's commands will not have
; to reset any previous state(s).
;------------------------------------------------------------------------------
;
_kill_leds
	clrf	red0
	clrf	red1
	clrf	red2
	clrf	red3
	clrf	red4
	clrf	red5
	clrf	grn0
	clrf	grn1
	clrf	grn2
	clrf	grn3
	clrf	grn4
	clrf	grn5
	return
;
;------------------------------------------------------------------------------
; Main loop
;
; A rate of ~59.2Hz is achieved at Fosc/4=250kHz clock and 22 cycles between
; LED activations and 16 level PWM structure.
; The actual LED activations are only at (22-5)/22 (77%) because of the TRIS
; plus PORT change taking 5 cycles.
;
; The interrupts to change the LEDs are scheduled to occur just before the
; charliplexing of the LEDs:
;
; +|----+|----+|----+|----+|----+|----+|----+|----+|----+|----+|----+|----
; Where:
;  '+' allows an interrupt
;  '|' charliplex to next LED
;  '-' delay
; The interrupt is rather long (~60 cycles fastest) with respect to the
; charliplex timing (22 cycles). Allowing the interrupt only to occur before
; the start of the change and disabling the active LED at the start of isr
; makes brightness changes less likely. The isr is only invoked 7.6 times per
; second. Each LED will receive ~7.75 activations (at correct brightness) for
; each timer interrupt.
;------------------------------------------------------------------------------
;
pausered	=	14 - charlicycles
pausegrn	=	30 - charlicycles
_loop
	charlipwm	red0, red0val, red0io
	incf	pwm, f		; Advance the PWM counter
	movlw	0x0f		; wrap count at 0..15
	andwf	pwm, f
	delaycycles	pausered - 3	; compensate for pwm increment
	
	charlipwm	red1, red1val, red1io
	movf	mode, w
	btfss	STATUS, Z	; If mode != 0
	goto	nosleep		;   goto nosleep
	movlw	0xff		; else set port all high
	movwf	PORTA		; and enable all pull-ups
	banksel	WPUA		; This is to minimize currents on floating inputs
	movwf	WPUA
	banksel	PORTA
	bsf	INTCON, GIE	; Make sure we get to the isr
	sleep
	movlw	(_modes_list_end - _modes_list) / 4 - 1
	movwf	mode		; Wakeup: immediately change mode
	movlw	HIGH(_alloff_table | 0x8000)
	movwf	current_starth
	movwf	FSR0H
	movlw	LOW(_alloff_table | 0x8000)
	movwf	current_startl
	movwf	FSR0L
	movlw	HIGH(_alloff_table_end - _alloff_table)
	movwf	ixcnth
	movlw	LOW(_alloff_table_end - _alloff_table)
	movwf	ixcntl
	movlw	0x07
	movwf	button		; and pretend we debounced and used it
	; Get here when woken up by button
nosleep
	delaycycles	pausered - 4	; compensate for mode test

_loop_entry
	charlipwm	red2, red2val, red2io
	delaycycles	pausered

	charlipwm	red3, red3val, red3io
	delaycycles	pausered

	charlipwm	red4, red4val, red4io
	delaycycles	pausered

	charlipwm	red5, red5val, red5io
	delaycycles	pausered

	charlipwm	grn0, grn0val, grn0io
	delaycycles	pausegrn

	charlipwm	grn1, grn1val, grn1io
	delaycycles	pausegrn

	charlipwm	grn2, grn2val, grn2io
	delaycycles	pausegrn

	charlipwm	grn3, grn3val, grn3io
	delaycycles	pausegrn

	charlipwm	grn4, grn4val, grn4io
	delaycycles	pausegrn

	charlipwm	grn5, grn5val, grn5io
	delaycycles	pausegrn - 2	; compensate for jump
	goto	_loop

;
;------------------------------------------------------------------------------
; Macros for data tables
; Format (in nibbles):
;  <led><value>
;
; - led 0..11
; - value 0..15
;------------------------------------------------------------------------------
;
blinkcmd	macro	__led, __val
	retlw	(((__led) & 0x0f) << 4) | ((__val) & 0x0f)
	endm

modeentry	macro	__tag, __tag_end
	retlw	LOW((__tag) | 0x8000)
	retlw	HIGH((__tag) | 0x8000)
	retlw	LOW((__tag_end) - (__tag))
	retlw	HIGH((__tag_end) - (__tag))
	endm

;
;------------------------------------------------------------------------------
; Data, data and more table data
_alloff_table
	blinkcmd	CMDALLYLW, 0
	blinkcmd	CMDNOP, 0	; need at least size 2
_alloff_table_end

;------------------------------------------------------------------------------
_redcircle_table
	blinkcmd	red0, 15
	blinkcmd	red5, 0
	blinkcmd	red1, 15
	blinkcmd	red0, 0
	blinkcmd	red2, 15
	blinkcmd	red1, 0
	blinkcmd	red3, 15
	blinkcmd	red2, 0
	blinkcmd	red4, 15
	blinkcmd	red3, 0
	blinkcmd	red5, 15
	blinkcmd	red4, 0
_redcircle_table_end

;------------------------------------------------------------------------------
_redcircle_rev_table
	blinkcmd	red5, 15
	blinkcmd	red0, 0
	blinkcmd	red4, 15
	blinkcmd	red5, 0
	blinkcmd	red3, 15
	blinkcmd	red4, 0
	blinkcmd	red2, 15
	blinkcmd	red3, 0
	blinkcmd	red1, 15
	blinkcmd	red2, 0
	blinkcmd	red0, 15
	blinkcmd	red1, 0
_redcircle_rev_table_end

;------------------------------------------------------------------------------
_grncircle_table
	blinkcmd	grn0, 15
	blinkcmd	grn5, 0
	blinkcmd	grn1, 15
	blinkcmd	grn0, 0
	blinkcmd	grn2, 15
	blinkcmd	grn1, 0
	blinkcmd	grn3, 15
	blinkcmd	grn2, 0
	blinkcmd	grn4, 15
	blinkcmd	grn3, 0
	blinkcmd	grn5, 15
	blinkcmd	grn4, 0
_grncircle_table_end

;------------------------------------------------------------------------------
_grncircle_rev_table
	blinkcmd	grn5, 15
	blinkcmd	grn0, 0
	blinkcmd	grn4, 15
	blinkcmd	grn5, 0
	blinkcmd	grn3, 15
	blinkcmd	grn4, 0
	blinkcmd	grn2, 15
	blinkcmd	grn3, 0
	blinkcmd	grn1, 15
	blinkcmd	grn2, 0
	blinkcmd	grn0, 15
	blinkcmd	grn1, 0
_grncircle_rev_table_end

;------------------------------------------------------------------------------
_ylwcircle_table
	blinkcmd	red0, 15
	blinkcmd	grn0, 15
	blinkcmd	red5, 0
	blinkcmd	grn5, 0
	blinkcmd	red1, 15
	blinkcmd	grn1, 15
	blinkcmd	red0, 0
	blinkcmd	grn0, 0
	blinkcmd	red2, 15
	blinkcmd	grn2, 15
	blinkcmd	red1, 0
	blinkcmd	grn1, 0
	blinkcmd	red3, 15
	blinkcmd	grn3, 15
	blinkcmd	red2, 0
	blinkcmd	grn2, 0
	blinkcmd	red4, 15
	blinkcmd	grn4, 15
	blinkcmd	red3, 0
	blinkcmd	grn3, 0
	blinkcmd	red5, 15
	blinkcmd	grn5, 15
	blinkcmd	red4, 0
	blinkcmd	grn4, 0
_ylwcircle_table_end

;------------------------------------------------------------------------------
_ylwcircle_rev_table
	blinkcmd	grn5, 15
	blinkcmd	red5, 15
	blinkcmd	grn0, 0
	blinkcmd	red0, 0
	blinkcmd	grn4, 15
	blinkcmd	red4, 15
	blinkcmd	grn5, 0
	blinkcmd	red5, 0
	blinkcmd	grn3, 15
	blinkcmd	red3, 15
	blinkcmd	grn4, 0
	blinkcmd	red4, 0
	blinkcmd	grn2, 15
	blinkcmd	red2, 15
	blinkcmd	grn3, 0
	blinkcmd	red3, 0
	blinkcmd	grn1, 15
	blinkcmd	red1, 15
	blinkcmd	grn2, 0
	blinkcmd	red2, 0
	blinkcmd	grn0, 15
	blinkcmd	red0, 15
	blinkcmd	grn1, 0
	blinkcmd	red1, 0
_ylwcircle_rev_table_end

;------------------------------------------------------------------------------
_smooth_table
cnt = 1
	while cnt < 16
	blinkcmd	CMDALLRED, cnt
cnt = cnt + 1
	endw
cnt = 1
	while cnt < 16
	blinkcmd	CMDALLGRN, cnt
cnt = cnt + 1
	endw
cnt = 14
	while cnt >= 0
	blinkcmd	CMDALLRED, cnt
cnt = cnt - 1
	endw
cnt = 14
	while cnt >= 0
	blinkcmd	CMDALLGRN, cnt
cnt = cnt - 1
	endw
_smooth_table_end

;------------------------------------------------------------------------------
_smooth_red_table
cnt = 1
	while cnt < 16
	blinkcmd	CMDALLRED, cnt
cnt = cnt + 1
	endw
cnt = 14
	while cnt >= 0
	blinkcmd	CMDALLRED, cnt
cnt = cnt - 1
	endw
_smooth_red_table_end

;------------------------------------------------------------------------------
_smooth_grn_table
cnt = 1
	while cnt < 16
	blinkcmd	CMDALLGRN, cnt
cnt = cnt + 1
	endw
cnt = 14
	while cnt >= 0
	blinkcmd	CMDALLGRN, cnt
cnt = cnt - 1
	endw
_smooth_grn_table_end

;------------------------------------------------------------------------------
_smooth_ylw_table
cnt = 1
	while cnt < 16
	blinkcmd	CMDALLYLW, cnt
cnt = cnt + 1
	endw
cnt = 14
	while cnt >= 0
	blinkcmd	CMDALLYLW, cnt
cnt = cnt - 1
	endw
_smooth_ylw_table_end

;------------------------------------------------------------------------------
_spinner_red_table
cnt = 1
	while cnt < 16
	blinkcmd	red0, cnt
	blinkcmd	red1, 15 - cnt
	blinkcmd	red3, cnt
	blinkcmd	red4, 15 - cnt
cnt = cnt + 1
	endw
cnt = 1
	while cnt < 16
	blinkcmd	red1, cnt
	blinkcmd	red2, 15 - cnt
	blinkcmd	red4, cnt
	blinkcmd	red5, 15 - cnt
cnt = cnt + 1
	endw
cnt = 1
	while cnt < 16
	blinkcmd	red2, cnt
	blinkcmd	red3, 15 - cnt
	blinkcmd	red5, cnt
	blinkcmd	red0, 15 - cnt
cnt = cnt + 1
	endw
_spinner_red_table_end

;------------------------------------------------------------------------------
_random_table
	blinkcmd	red5, 8
	blinkcmd	grn3, 11
	blinkcmd	grn5, 4
	blinkcmd	grn1, 5
	blinkcmd	red5, 15
	blinkcmd	red3, 14
	blinkcmd	grn3, 13
	blinkcmd	grn0, 3
	blinkcmd	red5, 10
	blinkcmd	red4, 8
	blinkcmd	grn1, 1
	blinkcmd	red1, 0
	blinkcmd	red3, 15
	blinkcmd	red1, 14
	blinkcmd	red4, 3
	blinkcmd	red1, 9
	blinkcmd	red3, 12
	blinkcmd	grn5, 1
	blinkcmd	grn5, 4
	blinkcmd	red1, 1
	blinkcmd	red4, 15
	blinkcmd	red1, 10
	blinkcmd	grn5, 1
	blinkcmd	grn4, 13
	blinkcmd	grn5, 8
	blinkcmd	grn5, 4
	blinkcmd	grn3, 15
	blinkcmd	grn1, 2
	blinkcmd	grn3, 5
	blinkcmd	red0, 13
	blinkcmd	red1, 6
	blinkcmd	grn0, 14
	blinkcmd	grn5, 9
	blinkcmd	red5, 2
	blinkcmd	red3, 14
	blinkcmd	grn0, 14
	blinkcmd	red1, 11
	blinkcmd	grn4, 3
	blinkcmd	grn1, 4
	blinkcmd	red0, 9
	blinkcmd	grn4, 15
	blinkcmd	red2, 6
	blinkcmd	grn5, 4
	blinkcmd	red4, 12
	blinkcmd	grn1, 9
	blinkcmd	red0, 14
	blinkcmd	red3, 11
	blinkcmd	red2, 13
	blinkcmd	red5, 6
	blinkcmd	red2, 3
	blinkcmd	grn1, 14
	blinkcmd	red5, 1
	blinkcmd	red0, 4
	blinkcmd	red3, 6
	blinkcmd	grn3, 1
	blinkcmd	red5, 13
	blinkcmd	grn5, 15
	blinkcmd	red5, 0
	blinkcmd	grn0, 8
	blinkcmd	grn5, 15
	blinkcmd	grn1, 10
	blinkcmd	red3, 13
	blinkcmd	grn1, 1
	blinkcmd	grn0, 5
	blinkcmd	red2, 0
	blinkcmd	red4, 4
	blinkcmd	red5, 9
	blinkcmd	red5, 13
	blinkcmd	grn3, 13
	blinkcmd	grn3, 6
	blinkcmd	red5, 2
	blinkcmd	grn3, 6
	blinkcmd	red0, 3
	blinkcmd	red0, 7
	blinkcmd	red1, 5
	blinkcmd	grn2, 12
	blinkcmd	grn4, 14
	blinkcmd	red5, 15
	blinkcmd	red5, 12
	blinkcmd	red5, 11
	blinkcmd	red1, 2
	blinkcmd	red0, 14
	blinkcmd	red2, 3
	blinkcmd	red2, 7
	blinkcmd	red1, 15
	blinkcmd	red4, 2
	blinkcmd	grn5, 12
	blinkcmd	grn2, 5
	blinkcmd	red3, 1
	blinkcmd	grn1, 11
	blinkcmd	grn3, 0
	blinkcmd	red0, 2
	blinkcmd	red5, 10
	blinkcmd	grn3, 9
	blinkcmd	grn5, 6
	blinkcmd	grn0, 14
	blinkcmd	grn0, 15
	blinkcmd	grn2, 12
	blinkcmd	red4, 0
	blinkcmd	grn1, 1
	blinkcmd	grn3, 9
	blinkcmd	grn4, 0
	blinkcmd	grn1, 0
	blinkcmd	grn0, 5
	blinkcmd	grn2, 12
	blinkcmd	grn2, 4
	blinkcmd	red1, 3
	blinkcmd	red3, 7
	blinkcmd	grn2, 3
	blinkcmd	red3, 6
	blinkcmd	red1, 13
	blinkcmd	grn2, 10
	blinkcmd	grn1, 11
	blinkcmd	red3, 3
	blinkcmd	red2, 1
	blinkcmd	grn1, 4
	blinkcmd	red0, 13
	blinkcmd	grn4, 0
	blinkcmd	red5, 5
	blinkcmd	grn1, 9
	blinkcmd	red4, 14
	blinkcmd	grn0, 0
	blinkcmd	grn3, 5
	blinkcmd	red5, 10
	blinkcmd	red1, 3
	blinkcmd	grn1, 13
	blinkcmd	grn2, 13
	blinkcmd	red3, 13
	blinkcmd	red3, 10
	blinkcmd	red5, 0
	blinkcmd	grn2, 1
	blinkcmd	red3, 13
	blinkcmd	red3, 2
	blinkcmd	grn5, 6
	blinkcmd	grn2, 0
	blinkcmd	grn0, 3
	blinkcmd	red3, 6
	blinkcmd	grn2, 7
	blinkcmd	grn4, 10
	blinkcmd	red0, 11
	blinkcmd	red5, 10
	blinkcmd	red1, 12
	blinkcmd	red5, 12
	blinkcmd	red4, 11
	blinkcmd	grn4, 1
	blinkcmd	grn4, 9
	blinkcmd	grn2, 11
	blinkcmd	grn2, 15
	blinkcmd	grn3, 13
	blinkcmd	red3, 4
	blinkcmd	grn0, 13
	blinkcmd	grn0, 1
	blinkcmd	red1, 0
	blinkcmd	grn1, 11
	blinkcmd	red3, 2
	blinkcmd	grn2, 4
	blinkcmd	grn2, 6
	blinkcmd	red5, 3
	blinkcmd	red1, 15
	blinkcmd	red5, 8
	blinkcmd	grn4, 15
	blinkcmd	red5, 9
	blinkcmd	red5, 7
	blinkcmd	grn4, 11
	blinkcmd	grn2, 13
	blinkcmd	grn2, 8
	blinkcmd	grn1, 6
	blinkcmd	grn1, 15
	blinkcmd	grn5, 7
	blinkcmd	grn3, 0
	blinkcmd	grn5, 6
	blinkcmd	red0, 10
	blinkcmd	grn5, 14
	blinkcmd	grn2, 6
	blinkcmd	red3, 12
	blinkcmd	red5, 4
	blinkcmd	grn1, 13
	blinkcmd	red3, 13
	blinkcmd	grn1, 5
	blinkcmd	red3, 3
	blinkcmd	grn0, 11
	blinkcmd	grn0, 8
	blinkcmd	red4, 0
	blinkcmd	red2, 7
	blinkcmd	grn0, 15
	blinkcmd	grn5, 12
	blinkcmd	grn1, 6
	blinkcmd	red0, 9
	blinkcmd	red0, 4
	blinkcmd	grn0, 0
	blinkcmd	grn4, 13
	blinkcmd	grn2, 9
	blinkcmd	red3, 6
	blinkcmd	grn0, 3
	blinkcmd	red3, 4
	blinkcmd	red0, 5
	blinkcmd	red3, 13
	blinkcmd	grn3, 10
	blinkcmd	red5, 9
	blinkcmd	red4, 12
	blinkcmd	grn4, 6
	blinkcmd	grn1, 13
	blinkcmd	red0, 3
	blinkcmd	red1, 9
	blinkcmd	grn0, 11
	blinkcmd	grn5, 15
	blinkcmd	red1, 9
	blinkcmd	red0, 10
	blinkcmd	red5, 11
	blinkcmd	grn3, 15
	blinkcmd	grn2, 12
	blinkcmd	grn1, 6
	blinkcmd	grn0, 5
	blinkcmd	red2, 5
	blinkcmd	red4, 8
	blinkcmd	grn1, 12
	blinkcmd	grn4, 2
	blinkcmd	red5, 11
	blinkcmd	red2, 2
	blinkcmd	red1, 8
	blinkcmd	red4, 5
	blinkcmd	red5, 6
	blinkcmd	grn2, 4
	blinkcmd	grn1, 12
	blinkcmd	grn1, 7
	blinkcmd	grn3, 3
	blinkcmd	red0, 11
	blinkcmd	red4, 14
	blinkcmd	grn1, 8
	blinkcmd	grn4, 0
	blinkcmd	grn2, 6
	blinkcmd	red3, 0
	blinkcmd	grn5, 3
	blinkcmd	red5, 0
	blinkcmd	red5, 9
	blinkcmd	grn3, 13
	blinkcmd	grn1, 11
	blinkcmd	grn3, 3
	blinkcmd	grn4, 3
	blinkcmd	grn2, 10
	blinkcmd	grn5, 10
	blinkcmd	red4, 15
	blinkcmd	red5, 10
	blinkcmd	red0, 15
	blinkcmd	grn4, 7
	blinkcmd	red2, 0
	blinkcmd	red1, 5
	blinkcmd	red5, 4
	blinkcmd	red1, 9
	blinkcmd	grn4, 9
	blinkcmd	red2, 7
	blinkcmd	red4, 8
	blinkcmd	grn3, 15
	blinkcmd	red1, 7
	blinkcmd	red3, 2
	blinkcmd	red1, 5
_random_table_end

;------------------------------------------------------------------------------
_wave_table
cnt = 1
	while cnt <= 5
	blinkcmd	red0, cnt
	blinkcmd	red1, cnt
cnt = cnt + 1
	endw
cnt = 1
	while cnt <= 5
	blinkcmd	red0, cnt+5
	blinkcmd	red1, cnt+5
	blinkcmd	grn0, cnt
	blinkcmd	grn1, cnt
	blinkcmd	red2, cnt
	blinkcmd	red5, cnt
cnt = cnt + 1
	endw
cnt = 1
	while cnt <= 5
	blinkcmd	red0, cnt+10
	blinkcmd	red1, cnt+10
	blinkcmd	grn0, cnt+5
	blinkcmd	grn1, cnt+5
	blinkcmd	red2, cnt+5
	blinkcmd	red5, cnt+5
	blinkcmd	grn2, cnt
	blinkcmd	grn5, cnt
	blinkcmd	red3, cnt
	blinkcmd	red4, cnt
cnt = cnt + 1
	endw
cnt = 1
	while cnt <= 5
	blinkcmd	red0, 15-cnt
	blinkcmd	red1, 15-cnt
	blinkcmd	grn0, cnt+10
	blinkcmd	grn1, cnt+10
	blinkcmd	red2, cnt+10
	blinkcmd	red5, cnt+10
	blinkcmd	grn2, cnt+5
	blinkcmd	grn5, cnt+5
	blinkcmd	red3, cnt+5
	blinkcmd	red4, cnt+5
	blinkcmd	grn3, cnt
	blinkcmd	grn4, cnt
cnt = cnt + 1
	endw
cnt = 1
	while cnt <= 5
	blinkcmd	red0, 10-cnt
	blinkcmd	red1, 10-cnt
	blinkcmd	grn0, 15-cnt
	blinkcmd	grn1, 15-cnt
	blinkcmd	red2, 15-cnt
	blinkcmd	red5, 15-cnt
	blinkcmd	grn2, cnt+10
	blinkcmd	grn5, cnt+10
	blinkcmd	red3, cnt+10
	blinkcmd	red4, cnt+10
	blinkcmd	grn3, cnt+5
	blinkcmd	grn4, cnt+5
cnt = cnt + 1
	endw
cnt = 1
	while cnt <= 5
	blinkcmd	red0, 5-cnt
	blinkcmd	red1, 5-cnt
	blinkcmd	grn0, 10-cnt
	blinkcmd	grn1, 10-cnt
	blinkcmd	red2, 10-cnt
	blinkcmd	red5, 10-cnt
	blinkcmd	grn2, 15-cnt
	blinkcmd	grn5, 15-cnt
	blinkcmd	red3, 15-cnt
	blinkcmd	red4, 15-cnt
	blinkcmd	grn3, cnt+10
	blinkcmd	grn4, cnt+10
cnt = cnt + 1
	endw
cnt = 1
	while cnt <= 5
	blinkcmd	grn0, 5-cnt
	blinkcmd	grn1, 5-cnt
	blinkcmd	red2, 5-cnt
	blinkcmd	red5, 5-cnt
	blinkcmd	grn2, 10-cnt
	blinkcmd	grn5, 10-cnt
	blinkcmd	red3, 10-cnt
	blinkcmd	red4, 10-cnt
	blinkcmd	grn3, 15-cnt
	blinkcmd	grn4, 15-cnt
cnt = cnt + 1
	endw
cnt = 1
	while cnt <= 5
	blinkcmd	grn2, 5-cnt
	blinkcmd	grn5, 5-cnt
	blinkcmd	red3, 5-cnt
	blinkcmd	red4, 5-cnt
	blinkcmd	grn3, 10-cnt
	blinkcmd	grn4, 10-cnt
cnt = cnt + 1
	endw
cnt = 1
	while cnt <= 5
	blinkcmd	grn3, 5-cnt
	blinkcmd	grn4, 5-cnt
cnt = cnt + 1
	endw
_wave_table_end

;------------------------------------------------------------------------------
_trailer_red_table
cnt2 = 0
	while cnt2 < 6
cnt = 0
	while cnt < 6
	blinkcmd	red0 + ((cnt + cnt2) % 6), (5-cnt) * 15 / 5
cnt = cnt + 1
	endw
cnt2 = cnt2 + 1
	endw
_trailer_red_table_end

;------------------------------------------------------------------------------
_trailer_grn_table
cnt2 = 0
	while cnt2 < 6
cnt = 0
	while cnt < 6
	blinkcmd	grn0 + ((cnt + cnt2) % 6), (5-cnt) * 15 / 5
cnt = cnt + 1
	endw
cnt2 = cnt2 + 1
	endw
_trailer_grn_table_end

;------------------------------------------------------------------------------
_ylw_2s_table
	blinkcmd	CMDALLYLW, 15	; 2s on
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDALLYLW, 0	; 2s off
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
_ylw_2s_table_end

;------------------------------------------------------------------------------
_ylw_2s_blink_table
	blinkcmd	CMDALLYLW, 15	; Blink once
	blinkcmd	CMDALLYLW, 0	; every 2s
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
	blinkcmd	CMDNOP, 0
_ylw_2s_blink_table_end

;------------------------------------------------------------------------------
_jumper_smooth_red_table
cnt = 0
	while cnt < 16
	blinkcmd	red0, cnt
	blinkcmd	red1, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	red3, cnt
	blinkcmd	red0, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	red5, cnt
	blinkcmd	red3, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	red2, cnt
	blinkcmd	red5, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	red4, cnt
	blinkcmd	red2, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	red1, cnt
	blinkcmd	red4, 15 - cnt
cnt = cnt + 3
	endw
_jumper_smooth_red_table_end

;------------------------------------------------------------------------------
_jumper_smooth_grn_table
cnt = 0
	while cnt < 16
	blinkcmd	grn0, cnt
	blinkcmd	grn1, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	grn3, cnt
	blinkcmd	grn0, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	grn5, cnt
	blinkcmd	grn3, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	grn2, cnt
	blinkcmd	grn5, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	grn4, cnt
	blinkcmd	grn2, 15 - cnt
cnt = cnt + 3
	endw
cnt = 0
	while cnt < 16
	blinkcmd	grn1, cnt
	blinkcmd	grn4, 15 - cnt
cnt = cnt + 3
	endw
_jumper_smooth_grn_table_end

;------------------------------------------------------------------------------
_sides_table
cnt = 0
	while cnt < 16
	blinkcmd	red2, cnt
	blinkcmd	grn5, cnt
	blinkcmd	red5, 15 - cnt
	blinkcmd	grn2, 15 - cnt
cnt = cnt + 1
	endw
cnt = 0
	while cnt < 16
	blinkcmd	red2, 15 - cnt
	blinkcmd	grn5, 15 - cnt
	blinkcmd	red5, cnt
	blinkcmd	grn2, cnt
cnt = cnt + 1
	endw
_sides_table_end
;
;------------------------------------------------------------------------------
; List of all modes
; has the ROM address and the mode's table-size
; Table is placed at end-of ROM
	org	0x800 - 21 * 4
_modes_list
	modeentry _alloff_table, _alloff_table_end
	modeentry _redcircle_table, _redcircle_table_end
	modeentry _redcircle_rev_table, _redcircle_rev_table_end
	modeentry _grncircle_table, _grncircle_table_end
	modeentry _grncircle_rev_table, _grncircle_rev_table_end
	modeentry _ylwcircle_table, _ylwcircle_table_end
	modeentry _ylwcircle_rev_table, _ylwcircle_rev_table_end
	modeentry _random_table, _random_table_end
	modeentry _sides_table, _sides_table_end
	modeentry _jumper_smooth_grn_table, _jumper_smooth_grn_table_end
	modeentry _jumper_smooth_red_table, _jumper_smooth_red_table_end
	modeentry _trailer_red_table, _trailer_red_table_end
	modeentry _trailer_grn_table, _trailer_grn_table_end
	modeentry _spinner_red_table, _spinner_red_table_end
	modeentry _wave_table, _wave_table_end
	modeentry _ylw_2s_table, _ylw_2s_table_end
	modeentry _ylw_2s_blink_table, _ylw_2s_blink_table_end
	modeentry _smooth_table, _smooth_table_end
	modeentry _smooth_grn_table, _smooth_grn_table_end
	modeentry _smooth_red_table, _smooth_red_table_end
	modeentry _smooth_ylw_table, _smooth_ylw_table_end
_modes_list_end

	end
